from typing import Any
try:
    from pymongo import MongoClient
except ImportError:
    raise ImportError("You must install pymongo to use MongoDB health check")

from .abstract import HealthCheckAbstract
from ..models import HealthCheckStatusEnum


class HealthCheckMongoDb(HealthCheckAbstract):
    def __init__(
        self,
        host: str | None,
        port: str | None,
        service: str | None = None,
        tags: list[str] | None = None,
        **kwargs: Any
    ):
        super().__init__(service=service, tags=tags)

        self.__host = host
        self.__port = port
        self.__kwargs = kwargs

    @property
    def service(self) -> str:
        return self._service if self._service is not None else "mongodb"

    @property
    def connection_uri(self) -> str | None:
        return f"mongodb://{self.__host}:{self.__port}" if self.__host and self.__port else None

    def check_health(self) -> HealthCheckStatusEnum:
        res: HealthCheckStatusEnum = HealthCheckStatusEnum.UNHEALTHY

        try:
            client = MongoClient(host=self.__host, port=self.__port, **self.__kwargs)
            client.server_info()
            res = HealthCheckStatusEnum.HEALTHY
        except Exception:
            pass

        return res



from typing import Callable, Any
from starlette.responses import JSONResponse
from fastapi.encoders import jsonable_encoder

from .registry import HealthCheckRegistry
from .models import HealthCheckStatusEnum


def health_check_route(registry: HealthCheckRegistry) -> Callable:
    """
    This function is passed to the add_api_route with the built factory.

    When called, the endpoint method within, will be called and it will run the job bound to the factory.
    The results will be parsed and sent back to the requestor via JSON.
    """

    def encode_json(value: Any) -> Any:
        """
        Encodes the provided value as a JSON column.
        :param value: the value to encode
        :return: the encoded value
        """
        return jsonable_encoder({} if value is None else value)

    def endpoint() -> JSONResponse:
        res = registry.check()
        if res["status"] == HealthCheckStatusEnum.UNHEALTHY:
            return JSONResponse(content=encode_json(res), status_code=500)
        return JSONResponse(content=encode_json(res), status_code=200)

    return endpoint



import asyncio
import time
import random
try:
    import aioredis
except ImportError:
    raise ImportError("You must install aioredis to use Redis health check")

from .abstract import HealthCheckAbstract
from ..models import HealthCheckStatusEnum


class HealthCheckRedis(HealthCheckAbstract):
    def __init__(self, uri: str, service: str | None = None, tags: list[str] | None = None):
        super().__init__(service=service, tags=tags)

        self.__uri = uri

    @property
    def service(self) -> str:
        return self._service if self._service is not None else "redis"

    @property
    def connection_uri(self) -> str | None:
        return self.__uri

    def check_health(self) -> HealthCheckStatusEnum:
        res: HealthCheckStatusEnum = HealthCheckStatusEnum.UNHEALTHY

        try:
            redis = aioredis.from_url(self.__uri)
            test_key = 'health_check_' + str(random.random()) + str(time.time())

            asyncio.run(redis.set(test_key, 'value'))
            result = asyncio.run(redis.get(test_key))

            asyncio.run(redis.delete(test_key))
            status = result == 'value'
            redis.close()
            asyncio.run(redis.wait_closed())

            if status:
                res = HealthCheckStatusEnum.HEALTHY
        except Exception:
            pass

        return res



from datetime import timedelta
from enum import Enum as BaseEnum
from pydantic import BaseModel


class HealthCheckStatusEnum(BaseEnum):
    HEALTHY = "Healthy"
    UNHEALTHY = "Unhealthy"


class HealthCheckEntityModel(BaseModel):
    service: str
    status: HealthCheckStatusEnum = HealthCheckStatusEnum.HEALTHY
    time_taken: timedelta | None = None
    tags: list[str] = []


class HealthCheckModel(BaseModel):
    status: HealthCheckStatusEnum = HealthCheckStatusEnum.HEALTHY
    total_time_taken: timedelta | None = None
    entities: list[HealthCheckEntityModel] = []



from .abstract import HealthCheckAbstract
from .mongodb import HealthCheckMongoDb
from .rabbitmq import HealthCheckRabbitMQ
from .database import HealthCheckDatabase
from .redis import HealthCheckRedis
from .uri import HealthCheckUri



try:
    from sqlalchemy import create_engine, text
    from sqlalchemy.orm import sessionmaker
except ImportError:
    raise ImportError("You must install sqlalchemy to use Database health check")

from .abstract import HealthCheckAbstract
from ..models import HealthCheckStatusEnum


class HealthCheckDatabase(HealthCheckAbstract):
    def __init__(self, uri: str, service: str | None = None, tags: list[str] | None = None):
        super().__init__(service=service, tags=tags)

        self.__uri = uri

    @property
    def service(self) -> str:
        return self._service if self._service is not None else "db"

    @property
    def connection_uri(self) -> str | None:
        return self.__uri

    def check_health(self) -> HealthCheckStatusEnum:
        res: HealthCheckStatusEnum = HealthCheckStatusEnum.UNHEALTHY

        engine = create_engine(self.__uri)
        Session = sessionmaker(autocommit=False, autoflush=False, bind=engine)
        with Session() as session:
            try:
                session.execute(text("SELECT 1"))
                res = HealthCheckStatusEnum.HEALTHY
            except Exception:
                pass
        return res
